#ifndef DAC_DRIVER_H
#define DAC_DRIVER_H

#include "avr_compiler.h"

// Enumerator for DAC channels.  
typedef enum DAC_CH_enum {
	CH0 = 0x00,
	CH1 = 0x01,
} DAC_CH_t;


// Enumerator for DAC event trigger.  
typedef enum DAC_TRIG_enum {
	DAC_TRIG_0_0 = 0x00,                                /*!< Trigger on no channel. */
	DAC_TRIG_0_1 = DAC_CH0TRIG_bm,                      /*!< Trigger on channel 0. */
	DAC_TRIG_1_0 = DAC_CH1TRIG_bm,                      /*!< Trigger on channel 1. */
	DAC_TRIG_1_1 = ( DAC_CH1TRIG_bm | DAC_CH0TRIG_bm ), /*!< Trigger on both channels. */
} DAC_TRIG_t;


/* Definition of macros. */

/*  This macro disables the selected DAC module.
 *
 *  \param  _dac  The DAC module, either DACA or DACB.
 */
#define DAC_Disable( _dac )                ( (_dac)->CTRLA &= ~DAC_ENABLE_bm )


/*  Enable output to internal ADC.
 *
 *  This macro enables routing of the internal DAC output to
 *  the ADC module. This feature is useful for diagnostics and
 *  calibration purposes.
 *
 *  \param  _dac  The DAC module, either DACA or DACB.
 */
#define DAC_InternalOutput_Enable( _dac )  ( (_dac)->CTRLA |= DAC_IDOEN_bm )


/*  This macro disables internal routing of the DAC output to an
 *         internal ADC module.
 *
 *  \param  _dac  The DAC module, either DACA or DACB.
 */
#define DAC_InternalOutput_Disable( _dac ) ( (_dac)->CTRLA &= ~DAC_IDOEN_bm )


// Prototyping of functions. Documentation is found in source file.  

void DAC_SingleChannel_Enable( volatile DAC_t * dac,
                               DAC_REFSEL_t convRef,
                               bool leftAdjust );
void DAC_DualChannel_Enable( volatile DAC_t * dac,
                             DAC_REFSEL_t convRef,
                             bool leftAdjust,
                             DAC_CONINTVAL_t sampleInterval,
                             DAC_REFRESH_t refreshInterval );
void DAC_Channel_Write( volatile DAC_t * dac, uint16_t data, DAC_CH_t channel );
bool DAC_Channel_DataEmpty( volatile DAC_t * dac, DAC_CH_t channel );
void DAC_EventAction_Set( volatile DAC_t * dac,
                          DAC_TRIG_t trigChannel,
                          uint8_t eventLine );

#endif

